package org.implab.gradle.containers.tasks;

import java.util.List;
import java.util.concurrent.Callable;

import org.gradle.api.provider.ListProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.Optional;
import org.implab.gradle.containers.ImageTag;

public abstract class PushImage extends CliTask {

    @Input
    public abstract Property<ImageTag> getSrcImage();

    @Input
    @Optional
    public abstract Property<ImageTag> getDestImage();

    @Input
    @Optional
    public abstract Property<String> getTransport();

    @Input
    @Optional
    public abstract ListProperty<String> getOptions();

    public PushImage option(Callable<String> provider) {
        getOptions().add(getProject().provider(provider));
        return this;
    }

    @Internal
    public Provider<String> getDestination() {
        return getDestImage().flatMap(tag -> getTransport().orElse("").map(t -> t + tag.toString()));
    }

    @Override
    protected void preparingCommand(List<String> commandLine) {
        super.preparingCommand(commandLine);

        commandLine.add("push");
        commandLine.addAll(getOptions().get());
        commandLine.add(getSrcImage().get().toString());
        if (getDestination().isPresent())
            commandLine.add(getDestination().get());
    }
}
